#! /usr/local/bin/apl --script
 ⍝ ********************************************************************
 ⍝   $Id: $
 ⍝ $desc: $
 ⍝ ********************************************************************

⍝ wp.apl, store, print manipulate an accountant's workpapers
⍝ Copyright (C) 2016  Bill Daly

⍝ This program is free software: you can redistribute it and/or modify
⍝ it under the terms of the GNU General Public License as published by
⍝ the Free Software Foundation, either version 3 of the License, or
⍝ (at your option) any later version.

⍝ This program is distributed in the hope that it will be useful,
⍝ but WITHOUT ANY WARRANTY; without even the implied warranty of
⍝ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
⍝ GNU General Public License for more details.

⍝ You should have received a copy of the GNU General Public License
⍝ along with this program.  If not, see <http://www.gnu.org/licenses/>.

)copy 1 cl

)copy 1 html_test

)copy 1 util

∇html←wp∆assemble wp;dat;size;hattr;ibattr;thead;tbody;hhead;attrs
 ⍝ Function to assemble an html page from a workpaper
 size←⍴dat←wp lex∆lookup 'Data'
 ⍝ Asemble table heading
 hattr←(lex∆init) lex∆assign 'class' 'page-head'
 hattr←hattr lex∆assign (⊂'colspan'),⊂ ⍕size[2]-1
 ibattr←(lex∆init) lex∆assign 'class' 'initial-block'
 lrattr←(lex∆init) lex∆assign 'class' 'last-head'
 thead←⎕av[11],html∆tr (hattr wp∆∆td wp lex∆lookup 'Entity'),ibattr wp∆∆td wp lex∆lookup 'Id'
 thead←thead,⎕av[11],html∆tr (hattr wp∆∆td wp lex∆lookup 'Title'),ibattr wp∆∆td wp lex∆lookup 'Author'
 thead←thead,⎕av[11],lrattr html∆tr (hattr wp∆∆td wp lex∆lookup 'Period'),ibattr wp∆∆td util∆today
 thead←html∆thead thead
 ⍝ Assemble body
 attrs←wp lex∆lookup 'Attributes'
 tbody←∊⎕av[11],¨html∆tr ¨ ∊¨ ⊂[2] size⍴ (,attrs) wp∆∆td ¨ ,dat
 ⍝ Assemble html head
 hhead←(html∆title wp lex∆lookup 'Title'),html∆style xml∆mkSheet wp lex∆lookup 'Stylesheet'
 hhead←hhead, html∆meta (lex∆init) lex∆assign 'charset' 'UTF8'
 hhead←html∆head hhead
 ⍝ Assemble document
 html←html∆html hhead, html∆body html∆table thead,tbody
∇

∇txt←wp∆exportTxt wp;shape;data;attr;nix
 ⍝ Function to create a tab delimited text array from a workpaper.
 shape←⍴data←wp lex∆lookup 'Data'
 data←,data
 attr←,wp lex∆lookup 'Attributes'
 nix←({⍵ lex∆haskey 'format'}¨attr)/⍳⍴attr
 data[nix]←({⍵ lex∆lookup 'format'}¨attr[nix])⍕¨data[nix]
 txt←(4 0+shape)⍴⊂' '
 txt[4+⍳shape[1];]←shape⍴data
 txt[1;1 2]←(⊂'Entity:'),⊂ wp lex∆lookup 'Entity'
 txt[2;1 2]←(⊂'Title:'),⊂ wp lex∆lookup 'Title'
 txt[3;1 2]←(⊂'Period:'),⊂ wp lex∆lookup 'Period'
 txt[1;shape[2]-1 0]←(⊂'Id'),⊂ wp lex∆lookup 'Id'
 txt[2;shape[2]-1 0]←(⊂'Author'),⊂ wp lex∆lookup 'Author'
 txt← ⎕av[11] util∆join ⎕av[10] util∆join ¨ ⊂[2]txt
∇

∇wp←wp∆importTxt fname;txt;shape;bv;data;meta
   ⍝ Function to import a tab delimted file as a workpaper
   wp←lex∆init
   txt←wp∆readPage fname
   shape←0 0
   shape[1]←+/1,bv←⎕av[11]=txt
   txt[bv/⍳⍴txt]←⎕av[10]
   shape[2]←(⍴ data←⎕av[10] util∆split txt)÷shape[1]
   data←shape⍴data
   meta←data[⍳4;]
   data←4 0↓data
   ⍝ Assume we're reading an export file we generated
   →(~∧/'Entity:'=⊃meta[1;1])/oh_well
   wp←wp lex∆assign (⊂'Entity'),meta[1;2]
   wp←wp lex∆assign (⊂'Title'),meta[2;2]
   wp←wp lex∆assign (⊂'Period'),meta[3;2]
   wp←wp lex∆assign (⊂'Id'),meta[1;shape[2]]
   wp←wp lex∆assign (⊂'Author'),meta[2;shape[2]]
   wp←wp lex∆assign (⊂'Attributes'),⊂(⍴data)⍴⊂lex∆init
   wp←wp lex∆assign (⊂'Stylesheet'),⊂lex∆init
   wp←wp lex∆assign (⊂'Data'),⊂data
   →0
 oh_well:
   ⍞←'Not a workpaper'
   →0
∇

∇new←meta wp∆init id;data;entity;title;period;auth;fn
 ⍝ Function to create a store of a workpaper's meta data
 →(2=⎕nc 'meta')/p0
 meta ←lex∆init
 p0:  				⍝ Remember id
 meta←meta lex∆assign (⊂'Id'),⊂id
 p1:				⍝ Prompt for data variable
 data←util∆prompt 'Data variable name:    '
 ⍎(util∆numberp data)/'→(qu,p1,dn,p1)[⎕io+data]'
 ⍎(2≠⎕nc data)/'⍞←data,'' does not exist in workspace. ''◊→p1'
 meta←meta lex∆assign (⊂'Data'), ⊂⍎data
 →p2
 ⍝⍞←⎕av[11]
 →p2
 p2:				⍝ Prompt for entity
 entity←util∆prompt 'Name of entity         '
 ⍎(util∆numberp entity)/'→(qu,p1,dn,p1)[⎕io+entity]'
 meta←meta lex∆assign 'Entity' entity
 ⍝⍞←⎕av[11]
 →p3
 p3:				⍝ Prompt for title
 title←util∆prompt  'Title of workpaper     '
 ⍎(util∆numberp title)/'→(qu,p1,dn,p2)[⎕io+title]'
 meta←meta lex∆assign 'Title' title
 ⍝⍞←⎕av[11]
 →p4
 p4:				⍝ Prompt for period
 period←util∆prompt 'Period of workpaper    '
 ⍎(util∆numberp period)/'→(qu,p1,dn,p3)[⎕io+period]'
 meta←meta lex∆assign 'Period' period
 ⍝⍞←⎕av[11]
 →p5
 p5:				⍝ Prompt for author
 auth←util∆prompt   'Author of workpaper    '
 ⍎(util∆numberp auth)/'→(qu,p1,dn,p4)[⎕io+period]'
 meta←meta lex∆assign 'Author' auth
 ⍝⍞←⎕av[11]
 →p6
 p6:		                ⍝ Prompt for function name
 fn←util∆prompt     'Function to calculate workpaper '
 ⍎(util∆numberp fn)/'→(qu,p1,dn,p5)[⎕io+fn]'
 meta←meta lex∆assign 'Function' fn
 ⍝⍞←⎕av[11]
 →p1
 dn:				⍝ Finish and exit
 new←meta
 →0
 qu:
 new←lex∆init
 →0
∇

∇txt←wp∆readPage fname;fh
 ⍝ Function to read (and return a text file).
 fh←'r' FIO∆fopen fname
 txt←⎕ucs FIO∆fread fh
 FIO∆fclose fh
∇

∇bytes←fname wp∆writePage page;fh
 ⍝ Function to write a page to a disk file.
 fh←'w' FIO∆fopen fname
 bytes←page FIO∆write_utf8 fh
 fh←FIO∆fclose fh
∇

∇var←wp∆∆extractData wp
 ⍝ Function returns the data array from a workpaper
 var←wp lex∆lookup 'Data'
∇

∇wp wp∆∆storeData data
 ⍝ Function stores its righ argument in the workpaper left argument
 wp lex∆assign data
∇

∇txt←attr wp∆∆td val
 ⍝ Function to format numeric data and return an html td element.
 →(attr lex∆haskey 'format')/hf
 txt←attr html∆td val
 →0
 hf:
 txt←(attr lex∆lookup 'format')⍕val
 txt←(attr lex∆drop 'format')html∆td txt
 →0
∇

cl∆∆max←20

